/****************************************************************************
**
** Copyright (C) 2019 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the demonstration applications of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:BSD$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** BSD License Usage
** Alternatively, you may use this file under the terms of the BSD license
** as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of The Qt Company Ltd nor the names of its
**     contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QtWidgets>

#include <emscripten/val.h>

// Save file function. Will be added to Qt 5.14 :)
void saveFileContent(const QString &fileName, const QByteArray &fileContent);

class LoadSaveWidget : public QWidget
{
public:
    LoadSaveWidget() {
        QVBoxLayout *layout = new QVBoxLayout();
        setLayout(layout);
        QHBoxLayout *buttonLayout = new QHBoxLayout();
        layout->addLayout(buttonLayout);
        QPushButton *loadFile = new QPushButton("Load File");
        buttonLayout->addWidget(loadFile);
        QPushButton *saveFile = new QPushButton("Save File");
        buttonLayout->addWidget(saveFile);
        QLabel *fileInfo = new QLabel("File");
        layout->addWidget(fileInfo);

        saveFile->setEnabled(false);
        
        auto fileReady = [=](const QString &name, const QByteArray &content) {
            fileName = name;
            fileContent = content;
            fileInfo->setText(QString("File: %1 Size: %2").arg(fileName).arg(fileContent.size()));
            saveFile->setEnabled(true);
        };
        
        QObject::connect(loadFile, &QPushButton::clicked, [=]() {
            QFileDialog::getOpenFileContent("*.*", fileReady);
        });
        QObject::connect(saveFile, &QPushButton::clicked, [=]() {
            saveFileContent(fileName, fileContent);
        });
    }

private:
    QString fileName;
    QByteArray fileContent;
};

void saveFileContent(const QString &fileName, const QByteArray &fileContent)
{
    auto saveFileImpl = [=]() {
        
        // Create file data Blob
        emscripten::val Blob = emscripten::val::global("Blob");
        emscripten::val content = emscripten::val(
            emscripten::typed_memory_view(fileContent.size(), fileContent.constData()));
        emscripten::val type = emscripten::val::object();
        type.set("type","octet/stream");
        emscripten::val fileBlob = Blob.new_(content, type);
    
        // Create Blob download link
        emscripten::val document = emscripten::val::global("document");
        emscripten::val link = document.call<emscripten::val>("createElement", std::string("a"));
        link.set("download", fileName.toStdString());
        emscripten::val window = emscripten::val::global("window");
        emscripten::val URL = window["URL"];
        emscripten::val objectUrl = URL.call<emscripten::val>("createObjectURL", fileBlob);
        link.set("href", objectUrl);
        link.set("style", "display:none");

        qDebug() << "savefileImpl 3";

        // Programatically click link
        emscripten::val body = document["body"];
        body.call<void>("appendChild", link);
        link.call<void>("click");

        // Clean up
        body.call<void>("removeChild", link);
        URL.call<void>("revokeObjectURL", objectUrl);
    };
    
    // Give the app some time to finish its current event processing,
    // before starting the potentially time consuming save.a
    QTimer::singleShot(10, saveFileImpl);
}

int main(int argc, char **argv)
{
    QApplication app(argc, argv);
    
    LoadSaveWidget loadSaveWidget;
    loadSaveWidget.showNormal();

    return app.exec();
}


